"""Image file profiles."""

from collections import UserDict

from rasterio.profiles import Profile  # type: ignore


class JPEGProfile(Profile):
    """JPEG creation options ref: https://www.gdal.org/frmt_jpeg.html."""

    defaults = {"quality": 85}


class PNGProfile(Profile):
    """PNG creation options ref: https://www.gdal.org/frmt_png.html."""

    defaults = {"zlevel": 6}


class PNGRAWProfile(Profile):
    """PNG creation options ref: https://www.gdal.org/frmt_png.html."""

    defaults = {"zlevel": 1}


class WEBPProfile(Profile):
    """WEBP creation options ref: https://www.gdal.org/frmt_webp.html."""

    defaults = {"quality": 75, "lossless": False}


class ImagesProfiles(UserDict):
    """GDAL Image creation options.

    ref: https://github.com/mapnik/mapnik/wiki/Image-IO#default-output-details.

    """

    def __init__(self):
        """Initialize ImagesProfiles dict."""
        self.data = {}
        self.data.update(
            {
                "jpeg": JPEGProfile(),
                "jpg": JPEGProfile(),
                "png": PNGProfile(),
                "pngraw": PNGRAWProfile(),
                "webp": WEBPProfile(),
            }
        )

    def get(self, key, default=None):
        """Like normal item access but return a copy of the key."""
        if key in (self.keys()):
            return self.data[key].copy()
        return default

    def __getitem__(self, key):
        """Like normal item access but return a copy of the key."""
        return self.data[key].copy()


img_profiles = ImagesProfiles()



"""rio-tiler.mosaic.methods abc class."""

import abc
from dataclasses import dataclass, field
from typing import Optional

import numpy


@dataclass
class MosaicMethodBase(abc.ABC):
    """Abstract base class for rio-tiler-mosaic methods objects."""

    mosaic: Optional[numpy.ma.MaskedArray] = field(default=None, init=False)
    exit_when_filled: bool = field(default=False, init=False)
    cutline_mask: Optional[numpy.ndarray] = field(default=None, init=False)
    width: int = field(init=False)
    height: int = field(init=False)
    count: int = field(init=False)

    @property
    def is_done(self) -> bool:
        """Check if the mosaic filling is done.

        Returns:
            bool

        """
        if self.mosaic is None:
            return False

        if self.exit_when_filled:
            if (
                self.cutline_mask is not None
                and numpy.sum(numpy.where(~self.cutline_mask, self.mosaic.mask, False))
                == 0
            ):
                return True
            elif not numpy.ma.is_masked(self.mosaic):
                return True

        return False

    @property
    def data(self) -> Optional[numpy.ma.MaskedArray]:
        """Return data."""
        return self.mosaic

    @abc.abstractmethod
    def feed(self, array: numpy.ma.MaskedArray):
        """Fill mosaic array.

        Args:
            array (numpy.ma.ndarray): data

        """



"""rio-tiler.expression: Parse and Apply expression."""

import re
from typing import List, Sequence, Tuple

import numexpr
import numpy

from rio_tiler.errors import InvalidExpression


def parse_expression(expression: str, cast: bool = True) -> Tuple:
    """Parse rio-tiler band math expression.

    Args:
        expression (str): band math/combination expression.
        cast (bool): cast band names to integers (convert to index values). Defaults to True.

    Returns:
        tuple: band names/indexes.

    Examples:
        >>> parse_expression("b1;b2")
            (2, 1)

        >>> parse_expression("B1/B2", cast=False)
            ("2", "1")

    """
    bands = set(re.findall(r"\bb(?P<bands>[0-9A-Z]+)\b", expression, re.IGNORECASE))
    output_bands = tuple(map(int, bands)) if cast else tuple(bands)
    if not output_bands:
        raise InvalidExpression(
            f"Could not find any valid bands in '{expression}' expression."
        )

    return output_bands


def get_expression_blocks(expression: str) -> List[str]:
    """Split expression in blocks.

    Args:
        expression (str): band math/combination expression.

    Returns:
        list: expression blocks (str).

    Examples:
        >>> parse_expression("b1/b2,b2+b1")
            ("b1/b2", "b2+b1")

    """
    return [expr for expr in expression.split(";") if expr]


def apply_expression(
    blocks: Sequence[str],
    bands: Sequence[str],
    data: numpy.ndarray,
) -> numpy.ma.MaskedArray:
    """Apply rio-tiler expression.

    Args:

        blocks (sequence): expression for a specific layer.
        bands (sequence): bands names.
        data (numpy.array):  array of bands.

    Returns:
        numpy.array: output data.

    """
    if len(bands) != data.shape[0]:
        raise ValueError(
            f"Incompatible number of bands ({bands}) and data shape {data.shape}"
        )

    try:
        return numpy.ma.MaskedArray(
            [
                numpy.nan_to_num(
                    numexpr.evaluate(bloc.strip(), local_dict=dict(zip(bands, data)))
                )
                for bloc in blocks
                if bloc
            ]
        )
    except KeyError as e:
        raise InvalidExpression(f"Invalid band/asset name {str(e)}") from e



"""rio-tiler.io"""

from .base import BaseReader, MultiBandReader, MultiBaseReader  # noqa
from .rasterio import ImageReader, Reader  # noqa
from .stac import STACReader  # noqa
from .xarray import XarrayReader  # noqa

# Keep Compatibility with <4.0
COGReader = Reader
